/* Color definition for portable colors */

/* Change those to fit to your MUD */

/* if you do not have color at all, just change all to be "" */

/* Foreground colors */

#define RED		""
#define BLUE		""
#define GREEN		""
#define BLACK		""
#define WHITE		""
#define MAGENTA		"" /* purple? */
#define YELLOW		""
#define CYAN		""

/* Background colors */

#define RED_BG			""
#define BLUE_BG			""
#define GREEN_BG		""
#define BLACK_BG		""
#define WHITE_BG		""
#define MAGENTA_BG		""
#define YELLOW_BG		""
#define CYAN_BG			""

/* Other codes */

#define BOLD			""
#define NO_COLOR		"" /* Default color */
#define REVERSE			""
#define TILDE           ""

